require_relative '../spec_helper'
require_relative '../../lib/scabox_sdk'

RSpec.describe 'Scanner' do
  describe 'check_info_flag' do
    context 'should print info and exit when info flag is true' do
      let(:scanner) { ScaBox::Scanner.new(name: 'test', description: 'test', support: []) }

      before do
        scanner.instance_variable_set(:@opts, OpenStruct.new(info: true))
      end

      it do
        expect { scanner.check_info_flag}.to raise_error(SystemExit) do |error|
          expect( error.status ).to eq(0)
        end
      end

      it do
        expect { scanner.check_info_flag}.to raise_error(SystemExit) do |error|
          expect( error.message ).to eq('exit')
        end
      end
    end
  end

  describe 'check_output_flag' do
    context 'should print error and exit when no output flag is provided' do
      let(:scanner) { ScaBox::Scanner.new(name: 'test', description: 'test', support: []) }

      before do
        scanner.instance_variable_set(:@opts, OpenStruct.new(output_stdout: false))
      end

      it do
        expect { scanner.check_output_flag }.to raise_error(SystemExit) do |error|
          expect( error.status ).to eq(1)
        end
      end

      it do
        expect { scanner.check_output_flag}.to raise_error(SystemExit) do |error|
          expect( error.message ).to eq('exit')
        end
      end
    end
  end

  describe 'start_scan' do
    context 'should raise name error for prepare_command method' do

      let(:scanner) { ScaBox::Scanner.new(name: 'test', description: 'test', support: []) }

      before do
        scanner.instance_variable_set(:@opts, OpenStruct.new())
      end

      it "should raise name error for prepare_command" do
        expect { scanner.run }.to raise_error(NameError) # #<NameError: undefined local variable or method `prepare_command' ...
      end
    end
  end


  describe 'add_issue' do
    context 'should add issue to instance array @issues' do
      let(:scanner) { ScaBox::Scanner.new(name: 'test', description: 'test', support: []) }

      before do

        scanner.add_issue(
          cve:              'CVE-XXXX-YYYY',
          references:       ['http://localhost'],
          title:            'title',
          discovery_method: 'discovery_method',
          description:      'description',
          severity:         'undefined',
          solution:         'solution'
        )
      end

      it "@issues.length == 1" do
        expect(scanner.issues.length).to eq 1
      end

      it "@issues[0][:plugin] == 'test'" do
        expect(scanner.issues[0][:plugin]).to eq 'test'
      end

      it "@issues[0][:cve] == 'CVE-XXXX-YYYY'" do
        expect(scanner.issues[0][:cve][0]).to eq 'CVE-XXXX-YYYY'
      end

      it "@issues[0][:references].length == 1" do
        expect(scanner.issues[0][:references].length).to eq 1
      end

      it "@issues[0][:title] == 'title'" do
        expect(scanner.issues[0][:title]).to eq 'title'
      end

      it "@issues[0][:discovery_method] == 'discovery_method'" do
        expect(scanner.issues[0][:discovery_method]).to eq 'discovery_method'
      end

      it "@issues[0][:description] == 'description'"  do
        expect(scanner.issues[0][:description]).to eq 'description'
      end

      it "@issues[0][:severity] == 'undefined'"  do
        expect(scanner.issues[0][:severity]).to eq 'undefined'
      end

      it "@issues[0][:solution] == 'solution'"  do
        expect(scanner.issues[0][:solution]).to eq 'solution'
      end

      it "@issues[0][:hash_issue] == 'e4634c7ab84ffe3a5ea189729f9b5a1763d527bf540d0512e87e5871045f7b4f'" do
        expect(scanner.issues[0][:hash_issue]).to eq 'e4634c7ab84ffe3a5ea189729f9b5a1763d527bf540d0512e87e5871045f7b4f'
      end

    end
  end

  describe 'build_title' do
    context 'should build title without CVE IDs' do
      let(:scanner) { ScaBox::Scanner.new(name: 'test', description: 'test', support: []) }

      before do
        scanner.instance_variable_set(:@opts, OpenStruct.new())
      end

      it "should build title without CVE IDs" do
        expect(scanner.build_title(cve: []) ).to eq 'Vulnerability'
      end
    end

    context 'should build title with CVE IDs' do
      let(:scanner) { ScaBox::Scanner.new(name: 'test', description: 'test', support: []) }

      before do
        scanner.instance_variable_set(:@opts, OpenStruct.new())
      end

      it "should build title with CVE IDs" do
        expect(scanner.build_title(cve: [4, 3, 2, 1]) ).to eq '1,2,3,4'
      end
    end
  end

  describe 'save_results' do
    context 'should save results to file' do
      let(:scanner) { ScaBox::Scanner.new(name: 'test', description: 'test', support: []) }
      let(:report) { '/tmp/save_results.json' }

      before do
        File.delete(report) if File.exist?(report)

        issue = {
          cve:              ['CVE-XXXX-YYYY'],
          references:       ['http://localhost'],
          title:            'title',
          discovery_method: 'discovery_method',
          description:      'description',
          severity:         'undefined',
          solution:         'solution'
        }

        scanner.instance_variable_set(:@issues, [issue])
        scanner.instance_variable_set(:@opts, OpenStruct.new(output: report))
        scanner.save_results
      end

      it "should save results to file" do
        expect(File.exist?(report)).to be true
      end
    end
  end

  describe 'txt_output' do
    context 'should format issue for txt output' do
      let(:scanner) { ScaBox::Scanner.new(name: 'test', description: 'test', support: []) }

      before do
        issue = {
          hash_issue:       'aaaaaaaaaaaaaaaaaaaaaa',
          plugin:           'plugin',
          path:             'lololol',
          component:        'xxx',
          version:          '123',
          cve:              ['CVE-XXXX-YYYY'],
          references:       ['http://localhost'],
          title:            'title',
          discovery_method: 'discovery_method',
          description:      'description',
          severity:         'undefined',
          solution:         'solution'
        }

        scanner.instance_variable_set(:@issues, [issue])
      end

      it "should format issue for txt output" do
        sep = "=" * 100
        output = ''

        output << "Hash Issue:         aaaaaaaaaaaaaaaaaaaaaa\n"
        output << "Plugin:             plugin\n"
        output << "Path:               lololol\n"
        output << "Component:          xxx\n"
        output << "Version:            123\n"
        output << "Discovery Method    discovery_method\n"
        output << "Title:              title\n"
        output << "Description:        description\n"
        output << "Solution:           solution\n"
        output << "Severity:           undefined\n"
        output << "CVE:                CVE-XXXX-YYYY\n"
        output << "References:\nhttp://localhost\n"
        output << "#{sep}\n\n"

        expect(scanner.txt_output ).to eq output
      end
    end
  end

  describe 'filename_for_plugin' do
    context 'should format filename for plugin' do
      let(:scanner) { ScaBox::Scanner.new(name: 'test', description: 'test', support: []) }

      it "should format filename for plugin" do
        expect(scanner.filename_for_plugin ).to match /test_.*\.json/
      end
    end
  end

  describe 'parse_json_from_str' do
    context 'should parse json from string' do
      let(:scanner) { ScaBox::Scanner.new(name: 'test', description: 'test', support: []) }

      it "should parse json from string" do
        expect(scanner.parse_json_from_str('{"abc": "123"}') ).to be_an Hash
        expect(scanner.parse_json_from_str('{"abc": "123"}') ).to have_key("abc")
        expect(scanner.parse_json_from_str('{"abc": "123"}')["abc"] ).to eq "123"
      end

      it "should return nil when json is invalid" do
        expect(scanner.parse_json_from_str('xxxxx') ).to eq nil
      end
    end
  end

  describe 'parse_json_from_file' do
    context 'should parse json from file' do
      let(:scanner) { ScaBox::Scanner.new(name: 'test', description: 'test', support: []) }
      let(:filename_valid) { '/tmp/json_input_valid_test.json' }
      let(:filename_invalid) { '/tmp/json_input_invalid_test.json' }

      before do
        File.write(filename_valid, '{"abc": "123"}')
        File.write(filename_invalid, 'xxxxx')
      end

      it "should parse json from file" do
        expect(scanner.parse_json_from_file(filename_valid) ).to be_an Hash
        expect(scanner.parse_json_from_file(filename_valid) ).to have_key("abc")
        expect(scanner.parse_json_from_file(filename_valid)["abc"] ).to eq "123"
      end

      it "should return nil when json is invalid" do
        expect(scanner.parse_json_from_file(filename_invalid) ).to eq nil
      end
    end
  end

  describe 'gen_random_tmp_filename' do
    context 'should generate random filename' do
      let(:scanner) { ScaBox::Scanner.new(name: 'test', description: 'test', support: []) }

      it "should generate random filename" do
        expect(scanner.gen_random_tmp_filename('.xxxyyy')).to match /\/tmp\/.*\.xxxyyy/
      end
    end
  end

  describe 'gen_random_tmp_filename_json' do
    context 'should generate random json filename' do
      let(:scanner) { ScaBox::Scanner.new(name: 'test', description: 'test', support: []) }

      it "should generate random json filename" do
        expect(scanner.gen_random_tmp_filename_json).to match /\/tmp\/.*\.json/
      end
    end
  end

end
